from collections import namedtuple

import gurobipy as g
import utils as utils
import sys

Point = namedtuple("Point", ['x', 'y'])
input_data = utils.load_problem(sys.argv)
points, node_count, cost_matrix = utils.preprocess(input_data)
big_m = sum(sum(cost_matrix[i][j] for i in range(node_count)) for j in range(node_count))

m = g.Model()

x = m.addVars(node_count, node_count, vtype="B")
s = m.addVars(node_count, vtype="C")

m.addConstrs(sum(x[i, j] for i in range(node_count) if i != j) == 1 for j in range(node_count))
m.addConstrs(sum(x[j, i] for i in range(node_count) if i != j) == 1 for j in range(node_count))
m.addConstrs(s[i] + cost_matrix[i][j] <= s[j] + big_m * (1 - x[i, j]) for i in range(node_count)
             for j in range(1, node_count))

m.setObjective(sum(cost_matrix[i][j] * x[i, j] for i in range(node_count) for j in range(node_count)))

m.optimize()

print(utils.output_sol(node_count, x, points))
